@@ -117,9 +117,39 @@ class Insight_Definition:
     def edit_widget(self, widget_id, widget_type: str, query_ids:list, field_codes:dict, values:dict={}, texts:dict={}, charts:list=[]):
 
         url = self.smart_insight_url+"/definitions/"+self.definition_id+"/widgets/"+widget_id
+        columns=[]
+        for query in query_ids:
+            if (not query.startswith("qcomp")):
 
-        columns = self.get_columns(query_ids)
+                url = self._base_url_askdata + "/smartinsight/definitions/" + self.definition_id + "/queries/" + query.lower() + "/columns"
+                r = self.__make_get_request(url)
+                cols = r.json()
 
+                while (cols == []):
+                    r = self.__make_get_request(url)
+                    cols = r.json()
+
+            else:
+                qc = self.get_component(query)
+
+                url_preview = self.smart_insight_url + "/composed_queries/" + qc["settings"][
+                    "qc"] + "/preview?limit=100&agentId=" + self.agent_id
+
+                s = requests.Session()
+                s.keep_alive = False
+                retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+                s.mount('https://', HTTPAdapter(max_retries=retries))
+
+                headers = {"Authorization": "Bearer" + " " + self._token}
+                response = s.post(url=url_preview, json={}, headers=headers)
+                response.raise_for_status()
+                r = response.json()
+                cols = r["schema"]
+
+            for col in cols:
+                columns.append(col)
+
+        print(columns)
         settings = {}
         for field in field_codes.keys():
             for column in columns:
@@ -135,6 +165,8 @@ class Insight_Definition:
                             settings[field + "_id"] = column["id"]
                             break
 
+        print(settings)
+
         for key in values.keys():
             settings[key + "_id"] = values[key]
 
